#! /bin/sh
#
# Generate 'tools/build/XXX.sh' from 'tools/src/XXX.sh.in'.
#
# Usage:
#     rewrite-tool.sh  target-file  KEY=VALUE ...
#
# Example:
#     rewrite-tool.sh  target-file  LIBDIR=/usr/local/lib  DISTDIR=$PWD/dist
#

PWD=`pwd`
TARGET_FILE=
LIBDIR=$PWD/lib
DISTDIR=$PWD/dist
LIBIDNKITLITEDIR=/usr/local/lib
TOOLSSRCDIR=$PWD/tools/src
TOOLSBUILDDIR=$PWD/tools/build

#
# Parse command line arguments.
#
while [ $# -gt 0 ]; do
    case "$1" in
    LIBDIR=*)
	LIBDIR=`echo $1 | sed -e 's/^[^=]*=//'`
	;;
    DISTDIR=*)
	DISTDIR=`echo $1 | sed -e 's/^[^=]*=//'`
	;;
    LIBIDNKITLITEDIR=*)
	LIBIDNKITLITEDIR=`echo $1 | sed -e 's/^[^=]*=//'`
	;;
    TOOLSBUILDDIR=*)
	TOOLSBUILDDIR=`echo $1 | sed -e 's/^[^=]*=//'`
	;;
    *=*)
	echo "$0: unknown keyword specified: $1" 1>&2
	exit 1
	;;
    *)
	if [ "X$TARGET_FILE" != X ]; then
	    echo "$0: multiple target files specified" 1>&2
	    exit 1
	fi
	TARGET_FILE=$1
	;;
    esac
    shift
done

if [ "X$TARGET_FILE" = X ]; then
    echo "$0: no target file specified" 1>&2
    exit 1
fi

#
# Convert "*DIR" values to absolute paths.
#
case "$LIBDIR" in
/*) ;;
*)  LIBDIR="$PWD/$LIBDIR"
esac

case "$DISTDIR" in
/*) ;;
*)  DISTDIR="$PWD/$DISTDIR"
esac

case "$TOOLSSRCDIR" in
/*) ;;
*)  TOOLSSRCDIR="$PWD/$TOOLSSRCDIR"
esac

case "$TOOLSBUILDDIR" in
/*) ;;
*)  TOOLSBUILDDIR="$PWD/$TOOLSBUILDDIR"
esac

#
# Find jar files.
#
JNAJAR=`ls -1 $LIBDIR/jna*.jar | head -1 2> /dev/null`
[ $? -eq 0 ] || JNAJAR=$LIBDIR/jna.jar

IDNKITJAR=`ls -1 $DISTDIR/idnkit*.jar | head -1 2> /dev/null`
[ $? -eq 0 ] || JNAJAR=$DISTDIR/idnkit.jar

#
# Rewrite the target file using *.in file.
#
if [ ! -f "$TOOLSSRCDIR/$TARGET_FILE.in" ]; then
    echo "$0: $TOOLSSRCDIR/$TARGET_FILE.in does not exist" 1>&2
    exit 1
fi

rm -f $TARGET_FILE
sed -e "s|@JNAJAR@|$JNAJAR|" \
    -e "s|@IDNKITJAR@|$IDNKITJAR|" \
    -e "s|@LIBIDNKITLITEDIR@|$LIBIDNKITLITEDIR|" \
    -e "s|@TOOLSBUILDDIR@|$TOOLSBUILDDIR|" \
    $TOOLSSRCDIR/$TARGET_FILE.in > $TOOLSBUILDDIR/$TARGET_FILE

chmod +x $TOOLSBUILDDIR/$TARGET_FILE

echo "$TOOLSBUILDDIR/$TARGET_FILE generated."
exit 0
