/*
 *                           TERMS AND CONDITIONS
 *                                   FOR
 *                         OPEN SOURCE CODE LICENSE
 *                               Version 1.1
 * 
 * Japan Registry Services Co., Ltd. ("JPRS"), a Japanese corporation
 * having its head office at Chiyoda First Bldg. East 13F 3-8-1 Nishi-Kanda,
 * Chiyoda-ku, Tokyo 101-0065, Japan, grants you the license for open source
 * code specified in EXHIBIT A the "Code" subject to the following Terms and
 * Conditions ("OSCL").
 * 
 * 1. License Grant.
 *   JPRS hereby grants you a worldwide, royalty-free, non-exclusive
 *   license, subject to third party intellectual property claims:
 *   (a) under intellectual property rights (other than patent or
 *       trademark) licensable by JPRS to use, reproduce, modify, display,
 *       perform, sublicense and distribute the Code (or portions thereof)
 *       with or without modifications, and/or as part of a derivative work;
 *       or
 *   (b) under claims of the infringement through the making, using,
 *       offering to sell and/or otherwise disposing the JPRS Revised Code
 *       (or portions thereof);
 *   (c) the licenses granted in this Section 1(a) and (b) are effective on
 *       the date JPRS first distributes the Code to you under the terms of
 *       this OSCL;
 *   (d) Notwithstanding the above stated terms, no patent license is
 *       granted:
 *       1)  for a code that you delete from the Code;
 *       2)  separate from the Code; or
 *       3)  for infringements caused by:
 *            i) modification of the Code; or
 *           ii) combination of the Code with other software or devices.
 * 
 * 2. Consents.
 *   You agree that:
 *   (a) you must include a copy of this OSCL and the notice set forth in
 *       EXHIBIT A with every copy of the Code you distribute;
 *   (b) you must include a copy of this OSCL and the notice set forth in
 *       EXHIBIT A with every copy of binary form of the Code in the
 *       documentation and/or other materials provided with the distribution;
 *   (c) you may not offer or impose any terms on any source code version
 *       that alters or restricts the applicable version of this OSCL or
 *       the recipients' rights hereunder.
 *   (d) If the terms and conditions are set forth in EXHIBIT A, you must
 *       comply with those terms and conditions.
 * 
 * 3. Proprietary Information.
 *   All trademarks, service marks, patents, copyrights, trade secrets, and
 *   other proprietary rights in or related to the Code are and will remain
 *   the exclusive property of JPRS or its licensors, whether or not
 *   specifically recognized or perfected under local law except specified
 *   in this OSCL; provided however you agree and understand that the JPRS
 *   name may not be used to endorse or promote this Code without prior
 *   written approval of JPRS.
 * 
 * 4. WARRANTY DISCLAIMER.
 *   JPRS MAKES NO REPRESENTATIONS AND WARRANTIES REGARDING THE USE OF THE
 *   CODE, NOR DOES JPRS MAKE ANY REPRESENTATIONS THAT THE CODE WILL BECOME
 *   COMMERCIALLY AVAILABLE. JPRS, ITS AFFILIATES, AND ITS SUPPLIERS DO NOT
 *   WARRANT OR REPRESENT THAT THE CODE IS FREE OF ERRORS OR THAT THE CODE
 *   IS SUITABLE FOR TRANSLATION AND/OR LOCALIZATION. THE CODE IS PROVIDED
 *   ON AN "AS IS" BASIS AND JPRS AND ITS SUPPLIERS HAVE NO OBLIGATION TO
 *   CORRECT ERRORS OR TO SUPPORT THE CODE UNDER THIS OSCL FOR ANY REASON.
 *   TO THE FULL EXTENT PERMITTED BY LAW, ALL OBLIGATIONS ARE HEREBY
 *   EXCLUDED WHETHER EXPRESS, STATUTORY OR IMPLIED UNDER LAW, COURSE OF
 *   DEALING, CUSTOM, TRADE USAGE, ORAL OR WRITTEN STATEMENT OR OTHERWISE,
 *   INCLUDING BUT NOT LIMITED TO ANY IMPLIED WARRANTIES OF MERCHANTABILITY
 *   OR FITNESS FOR A PARTICULAR PURPOSE CONCERNING THE CODE.
 * 
 * 5. NO LIABILITY.
 *   UNDER NO CIRCUMSTANCES SHALL JPRS AND/OR ITS AFFILIATES, LICENSORS, OR
 *   REPRESENTATIVES BE LIABLE FOR ANY DAMAGES INCLUDING BUT NOT LIMITED TO
 *   CONSEQUENTIAL, INDIRECT, SPECIAL, PUNITIVE OR INCIDENTAL DAMAGES,
 *   WHETHER FORESEEABLE OR UNFORESEEABLE, BASED ON YOUR CLAIMS, INCLUDING,
 *   BUT NOT LIMITED TO, CLAIMS FOR LOSS OF DATA, GOODWILL, PROFITS, USE OF
 *   MONEY, INTERRUPTION IN USE OR AVAILABILITY OF DATA, STOPPAGE, IMPLIED
 *   WARRANTY, BREACH OF CONTRACT, MISREPRESENTATION, NEGLIGENCE, STRICT
 *   LIABILITY IN TORT, OR OTHERWISE.
 * 
 * 6. Indemnification.
 *   You hereby agree to indemnify, defend, and hold harmless JPRS for any
 *   liability incurred by JRPS due to your terms of warranty, support,
 *   indemnity, or liability offered by you to any third party.
 * 
 * 7. Termination.
 * 7.1 This OSCL shall be automatically terminated in the events that:
 *   (a) You fail to comply with the terms herein and fail to cure such
 *       breach within 30 days of becoming aware of the breach;
 *   (b) You initiate patent or copyright infringement litigation against
 *       any party (including a cross-claim or counterclaim in a lawsuit)
 *       alleging that the Code constitutes a direct or indirect patent or
 *       copyright infringement, in such case, this OSCL to you shall
 *       terminate as of the date such litigation is filed;
 * 7.2 In the event of termination under Sections 7.1(a) or 7.1(b) above,
 *     all end user license agreements (excluding distributors and
 *     resellers) which have been validly granted by You or any distributor
 *     hereunder prior to termination shall survive termination.
 *
 * 
 * 8. General.
 *   This OSCL shall be governed by, and construed and enforced in
 *   accordance with, the laws of Japan. Any litigation or arbitration
 *   between the parties shall be conducted exclusively in Tokyo, Japan
 *   except written consent of JPRS provides other venue.
 * 
 * 
 *                                EXHIBIT A
 * 
 * The original open source code of idnkit-2 is idnkit-1.0 developed and
 * conceived by Japan Network Information Center ("JPNIC"), a Japanese
 * association, Kokusai-Kougyou-Kanda Bldg 6F, 2-3-4 Uchi-Kanda,
 * Chiyoda-ku, Tokyo 101-0047, Japan, and JPRS modifies above original code
 * under following Terms and Conditions set forth by JPNIC.
 * 
 *                                  JPNIC
 * 
 * Copyright (c) 2000-2002 Japan Network Information Center.  All rights reserved.
 * 
 * By using this file, you agree to the terms and conditions set forth bellow.
 * 
 *                       LICENSE TERMS AND CONDITIONS
 * 
 * The following License Terms and Conditions apply, unless a different
 * license is obtained from Japan Network Information Center ("JPNIC"),
 * a Japanese association, Kokusai-Kougyou-Kanda Bldg 6F, 2-3-4 Uchi-Kanda,
 * Chiyoda-ku, Tokyo 101-0047, Japan.
 * 
 * 1. Use, Modification and Redistribution (including distribution of any
 *    modified or derived work) in source and/or binary forms is permitted
 *    under this License Terms and Conditions.
 * 
 * 2. Redistribution of source code must retain the copyright notices as they
 *    appear in each source code file, this License Terms and Conditions.
 * 
 * 3. Redistribution in binary form must reproduce the Copyright Notice,
 *    this License Terms and Conditions, in the documentation and/or other
 *    materials provided with the distribution. For the purposes of binary
 *    distribution the "Copyright Notice" refers to the following language:
 *    "Copyright (c) 2000-2002 Japan Network Information Center.  All rights reserved."
 * 
 * 4. The name of JPNIC may not be used to endorse or promote products
 *    derived from this Software without specific prior written approval of
 *    JPNIC.
 * 
 * 5. Disclaimer/Limitation of Liability: THIS SOFTWARE IS PROVIDED BY JPNIC
 *    "AS IS" AND ANY EXPRESS OR IMPLIED WARRANTIES, INCLUDING, BUT NOT
 *    LIMITED TO, THE IMPLIED WARRANTIES OF MERCHANTABILITY AND FITNESS FOR A
 *    PARTICULAR PURPOSE ARE DISCLAIMED. IN NO EVENT SHALL JPNIC BE LIABLE
 *    FOR ANY DIRECT, INDIRECT, INCIDENTAL, SPECIAL, EXEMPLARY, OR
 *    CONSEQUENTIAL DAMAGES (INCLUDING, BUT NOT LIMITED TO, PROCUREMENT OF
 *    SUBSTITUTE GOODS OR SERVICES; LOSS OF USE, DATA, OR PROFITS; OR
 *    BUSINESS INTERRUPTION) HOWEVER CAUSED AND ON ANY THEORY OF LIABILITY,
 *    WHETHER IN CONTRACT, STRICT LIABILITY, OR TORT (INCLUDING NEGLIGENCE OR
 *    OTHERWISE) ARISING IN ANY WAY OUT OF THE USE OF THIS SOFTWARE, EVEN IF
 *    ADVISED OF THE POSSIBILITY OF SUCH DAMAGES.
 * 
 * 
 *                        JPRS Public License Notice
 *                                   For
 *                                idnkit-2.
 * 
 * The contents of this file are subject to the Terms and Conditions for
 * the Open Source Code License (the "OSCL"). You may not use this file
 * except in compliance with above terms and conditions. A copy of the OSCL
 * is available at <http://jprs.co.jp/idn/>.
 * The JPRS Revised Code is idnkit-2.
 * The Initial Developer of the JPRS Revised Code is Japan Network
 * Information Center ("JPNIC"), a Japanese association,
 * Kokusai-Kougyou-Kanda Bldg 6F, 2-3-4 Uchi-Kanda, Chiyoda-ku, Tokyo
 * 101-0047, Japan.
 * "Copyright (c) 2000-2002 Japan Network Information Center.  All rights reserved."
 * "Copyright (c) 2010-2012 Japan Registry Services Co., Ltd.  All rights reserved."
 * Contributor(s): ______________________________________.
 * 
 * If you wish to allow use of your version of this file only under the
 * above License(s) and not to allow others to use your version of this
 * file, please indicate your decision by deleting the relevant provisions
 * above and replacing them with the notice and other provisions required
 * by the above License(s). If you do not delete the relevant provisions,
 * a recipient may use your version of this file under either the above
 * License(s).
 */
package jp.co.jprs.idnkit;

/**
 * <p>
 * The jp.co.jprs.idnkit.IDNA interface defines manipulations of
 * internationalized domain names using idnkit.
 * </p>
 * 
 * <p>
 * This interface defines almost the same methods as idnkit library
 * provides.  Its main functions are encoding (conversion to ASCII)
 * and decoding (conversion to UTF-8) domain names.
 * </p>
 * 
 * <p>
 * Currently, the package provides IDNA2008 class as an implementation
 * of the jp.co.jprs.idnkit.IDNA interface.  To create an IDNA2008
 * instance, IDNAFactory class is used instead of a constructor of
 * IDNA2008 class.
 * </p>
 * 
 * @author Japan Registry Services Co., Ltd.(JPRS)
 * @version 2.2
 * @see jp.co.jprs.idnkit.IDNAFactory
 * @see jp.co.jprs.idnkit.IDNA2008
 */
public interface IDNA {
    /**
     * Action value to encode a domain name with the domain name registration
     * protocol.
     * Also local-check is performed if the conversion context has
     * code point set.
     */
    static final int ENCODE_REGIST = -1;

    /**
     * Action value to encode a domain name with the domain name lookup
     * protocol.
     * Also local-check is performed if the conversion context has
     * code point set.
     */
    static final int ENCODE_LOOKUP = -2;

    /**
     * Action value to decode a domain name with the domain name registration
     * protocol.
     * Also local-check is performed if the conversion context has
     * code point set.
     */
    static final int DECODE_REGIST = -3;

    /**
     * Action value to decode a domain name with the domain name lookup
     * protocol.
     * Also local-check is performed if the conversion context has
     * code point set.
     */
    static final int DECODE_LOOKUP = -4;

    /**
     * Action value to compare two domain names with the domain name
     * registration protocol.
     * Also local-check is performed if the conversion context has
     * code point set.
     */
    static final int COMPARE_REGIST = ENCODE_REGIST;

    /**
     * Action value to compare two domain names with the domain name lookup
     * protocol.
     * Also local-check is performed if the conversion context has
     * code point set.
     */
    static final int COMPARE_LOOKUP = ENCODE_LOOKUP;

    /**
     * Action value to validate a domain name with the domain name
     * registration protocol.
     * Also local-check is performed if the conversion context has
     * code point set.
     */
    static final int CHECK_REGIST   = ENCODE_REGIST;

    /**
     * Action value to validate a domain name with the domain name lookup
     * protocol.
     * Also local-check is performed if the conversion context has
     * code point set.
     */
    static final int CHECK_LOOKUP   = ENCODE_LOOKUP;

    /**
     * Initializes the conversion context instance.
     * Upon return, the instance will have the default settings.
     *
     * @exception IDNException  Initialization failed.
     */
    public void init() throws IDNException;

    /**
     * Initializes the conversion context instance.
     * The context will be initialized according with the configuration
     * file specified by the argument 'filename'.
     * 
     * @param filename  Configuration file to be read.
     * @exception IDNException  Initialization failed.
     */
    public void init(String filename) throws IDNException;

    /**
     * Initializes the conversion context instance.
     * The context will be initialized according with the configuration
     * strings specified by the argument 'conf'.
     * 
     * @param conf  Configurations to be set.
     * @exception IDNException  Initialization failed.
     */
    public void init(String[] conf) throws IDNException;

    /**
     * Reads a code point set for local-check from a file 'filename'.
     * If 'filename' is null, the local code point set the conversion
     * context memories is cleared.
     * 
     * @param filename  Code point set file to be read.
     * @exception IDNException  Failed to read the file.
     */
    public void setLocalCheckFile(String filename) throws IDNException;

    /**
     * Converts a domain name 'name' to ASCII, and returns the result.
     * The method performs ENCODE_REGIST action to convert 'name'.
     *
     * @param name  A domain name to be encoded.
     * @return An encoded domain name.
     * @exception IDNException  Conversion failed.
     */
    public String encode(String name) throws IDNException;

    /**
     * Converts a domain name 'name' to ASCII and returns the result.
     * The argument 'actions' specifies which steps in the entire encoding
     * process should be performed.
     * 
     * @param actions  An action value to encode 'name'.
     * @param name  A domain name to be encoded.
     * @return An encoded domain name.
     * @exception IDNException  Conversion failed.
     */
    public String encode(int actions, String name) throws IDNException;

    /**
     * Converts a domain name 'name' to UTF-8 and returns the result.
     * The method performs DECODE_REGIST action to convert 'name'.
     * 
     * @param name  A domain name to be decoded.
     * @return A decoded domain name.
     * @exception IDNException  Conversion failed.
     */
    public String decode(String name) throws IDNException;

    /**
     * Converts a domain name 'name' to UTF-8 and returns the result.
     * The argument 'actions' specifies which steps in the entire decoding
     * process should be performed.
     * 
     * @param actions  An action value to decode 'name'.
     * @param name  A domain name to be decoded.
     * @return A decoded domain name.
     * @exception IDNException  Conversion failed.
     */
    public String decode(int actions, String name) throws IDNException;

    /**
     * Compares two domain names.
     * It encodes domain names 'name1' and 'name2' as the 'encode' method
     * does, and then compares them.  If they are equivalent, it returns
     * true.  Otherwise, it returns false.
     * The method performs COMPARE_LOOKUP action to encode 'name1' and
     * 'name2'.
     * 
     * @param name1  A domain name #1.
     * @param name2  A domain name #2.
     * @exception IDNException  Conversion of domain names failed.
     */
    public boolean compare(String name1, String name2) throws IDNException;

    /**
     * Compares two domain names.
     * It encodes domain names 'name1' and 'name2' as the 'encode' method
     * does, and then compares them.  If they are equivalent, it returns
     * true.  Otherwise, it returns false.
     * The method performs 'actions' to encode 'name1' and 'name2'.
     * 
     * @param actions  An Action value to encode 'name1' and 'name2'.
     * @param name1  A domain name #1.
     * @param name2  A domain name #2.
     * @return True if two domain names are equivalent.
     * @exception IDNException  Conversion of domain names failed.
     */
    public boolean compare(int actions, String name1, String name2)
	throws IDNException;

    /**
     * Compares two domain names.
     * It encodes domain names 'name1' and 'name2' as the 'encode' method
     * does, and then compares them.  If they are equivalent, it returns
     * true.  Otherwise, it returns false.
     * The method performs 'actions1' to encode 'name1', and 'actions2'
     * to encode 'name2' respectively.
     * 
     * @param actions1  An Action value to encode 'name1'.
     * @param name1  A domain name #1.
     * @param actions2  An Action value to encode 'name2'.
     * @param name2  A domain name #2.
     * @return True if two domain names are equivalent.
     * @exception IDNException  Conversion of domain names failed.
     */
    public boolean compare(int actions1, String name1, int actions2,
	String name2) throws IDNException;

    /**
     * Validates a domain name.
     * It tries to encode 'name' with CHECK_REGIST action.
     * If succeeded, it returns true.  Otherwise, it returns false.
     * 
     * @param name  A domain name to be validated.
     * @return True if the domain name is valid.
     * @exception IDNException  Conversion of domain names failed.
     */
    public boolean check(String name) throws IDNException;

    /**
     * Validates a domain name.
     * It tries to encode 'name' with an action value 'actions'.
     * If succeeded, it returns true. Otherwise, it returns false.
     * 
     * @param name  A domain name to be validated.
     * @param actions  An Action value to encode 'name'.
     * @return True if the domain name is valid.
     * @exception IDNException  Conversion of domain names failed.
     */
    public boolean check(int actions, String name) throws IDNException;

    /**
     * Gets a version of Unicode tables the context instance supports.
     *
     * @return Version (e.g. "6.0.0" in case of Unicode 6.0.0).
     */
    public String getUnicodeVersion();

    /**
     * Gets a version of IDNA tables the context instance supports.
     * 
     * @return Version (e.g. "rfc6452 (2011-11-14)").
     */
    public String getIdnaTableVersion();
    
    /**
     * Gets a version of idnkit library.
     * 
     * @return Version (e.g. "2.2" for idnkit version 2.2).
     */
    public String getLibIDNVersion();

    /**
     * Returns version information.
     * It returns a string which consists of a version of Unicode tables,
     * a version of IDNA tables and a version of idnkit library.
     * 
     * @return Version string.
     */
    public String getVersion();
}
